#Region "Creation Filter Classes"

Public Class MonthViewSingleAppointmentCreationFilter
    Implements IUIElementCreationFilter

    Private DayApps As New Dictionary(Of Appointment, SingleDayAppointmentUIElement)()
    Private ulMonthViewSingle As Infragistics.Win.UltraWinSchedule.UltraMonthViewSingle

    Public Sub New(ByVal ultraMonthViewSingle1 As Infragistics.Win.UltraWinSchedule.UltraMonthViewSingle)
        Me.ulMonthViewSingle = ultraMonthViewSingle1
        ultraMonthViewSingle1.DrawFilter = New MonthViewSingleAppointmentDrawFilter(DayApps)
    End Sub

    Public Sub AfterCreateChildElements(ByVal parent As UIElement) Implements IUIElementCreationFilter.AfterCreateChildElements
        Dim objDayNumberUIElement As DayNumberUIElement = TryCast(parent, DayNumberUIElement)
        If objDayNumberUIElement IsNot Nothing Then
            Dim objDayElement As DayUIElement = DirectCast(objDayNumberUIElement.Parent, DayUIElement)
            If objDayElement IsNot Nothing Then
                If objDayElement.Rect.Height <= (objDayElement.Day.Appointments.Count * 67) Then
                    Dim mIndicator As New Office2007MoreActivityIndicatorUIElement(objDayNumberUIElement, DirectCast(objDayNumberUIElement.Parent, DayUIElement).[Date])
                    mIndicator.Rect = New Rectangle(objDayNumberUIElement.Rect.X + objDayNumberUIElement.Rect.Width - 15, objDayNumberUIElement.Rect.Y + 4, 10, 10)
                    If objDayElement IsNot Nothing Then
                        objDayNumberUIElement.ChildElements.Add(mIndicator)
                    End If
                End If

                Dim objWeekUIElement = objDayElement.GetAncestor(GetType(WeekUIElement))
                If objWeekUIElement.GetAncestor(GetType(MonthAreaUIElement)) IsNot Nothing AndAlso TypeOf objWeekUIElement.GetAncestor(GetType(MonthAreaUIElement)) Is MonthAreaUIElement Then
                    For Each rChild In objWeekUIElement.ChildElements
                        If TypeOf rChild Is Office2007MoreActivityIndicatorUIElement Then
                            objWeekUIElement.ChildElements.RemoveAt(objWeekUIElement.ChildElements.IndexOf(rChild))
                        End If
                    Next
                End If
            End If
        End If
    End Sub

    Public Function BeforeCreateChildElements(ByVal parent As UIElement) As Boolean Implements IUIElementCreationFilter.BeforeCreateChildElements
        Dim dayHeader As DayNumberUIElement = TryCast(parent, DayNumberUIElement)
        If dayHeader IsNot Nothing Then
            Dim day As DayUIElement = TryCast(dayHeader.Parent, DayUIElement)

            If day.Day.Appointments.Count = 0 Then
                Return False
            End If

            Dim AppAreaLocation As Point = dayHeader.Rect.Location
            AppAreaLocation.Offset(0, dayHeader.Rect.Height + 2)

            Dim sFirstHalf As String = String.Empty
            Dim sSecondHalf As String = String.Empty

            sFirstHalf = day.Day.Appointments(0).Text.Substring(0, day.Day.Appointments(0).Text.Split(" ")(0).Length + 3).Trim()
            sSecondHalf = day.Day.Appointments(0).Text.Substring(sFirstHalf.Length, day.Day.Appointments(0).Text.Length - sFirstHalf.Length).Trim()
            day.Day.Appointments(0).Subject = sFirstHalf + vbCrLf + sSecondHalf

            Dim objFirstApp As New SingleDayAppointmentUIElement(day, day.Day.Appointments(0))
            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(day.Day.Appointments(0).Subject, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 30, CInt(objSizeF.Height))
            Dim appSize As Size

            If iHeight + 4 > day.Rect.Height OrElse day.Rect.Height < 24 Then
                iHeight = day.Rect.Height - 4
            End If

            If day.Rect.Width < 95 Then
                appSize = New Size(day.Rect.Width - (day.Rect.Width * 0.2), iHeight)
            Else
                appSize = New Size(IIf(objSizeF.Width > 90, 90, objSizeF.Width), iHeight)
            End If

            AppAreaLocation = New Point(dayHeader.Rect.X + (dayHeader.Rect.Width / 2) - (appSize.Width / 2), AppAreaLocation.Y)

            Dim AppArea As New Size(dayHeader.Rect.Width, day.Rect.Height - dayHeader.Rect.Height - 2)
            Dim appLocation As Point = AppAreaLocation
            Dim newAppUI As New List(Of SingleDayAppointmentUIElement)()

            For Each app As Appointment In day.Day.Appointments
                Dim appUI As New SingleDayAppointmentUIElement(day, app)

                sFirstHalf = app.Text.Substring(0, app.Text.Split(" ")(0).Length + 3).Trim()
                sSecondHalf = app.Text.Substring(sFirstHalf.Length, app.Text.Length - sFirstHalf.Length).Trim()
                app.Subject = sFirstHalf + vbCrLf + sSecondHalf

                objAppFont = New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
                objSizeF = TextRenderer.MeasureText(app.Subject, objAppFont)
                iHeight = IIf(CInt(objSizeF.Height) <= 0, 30, CInt(objSizeF.Height))

                If iHeight + 4 > day.Rect.Height OrElse day.Rect.Height < 24 Then
                    iHeight = day.Rect.Height - 4
                End If

                If day.Rect.Width < 95 Then
                    appSize = New Size(day.Rect.Width - (day.Rect.Width * 0.2), iHeight - 10)
                Else
                    appSize = New Size(IIf(objSizeF.Width > 90, 90, objSizeF.Width), iHeight - 10)
                End If

                AppAreaLocation = New Point(dayHeader.Rect.X + (dayHeader.Rect.Width / 2) - (appSize.Width / 2), AppAreaLocation.Y)
                appLocation = New Point(appLocation.X, appLocation.Y)
                appUI.Rect = New Rectangle(appLocation, appSize)

                If DayApps.ContainsKey(app) Then
                    DayApps(app) = appUI
                Else
                    DayApps.Add(app, appUI)
                End If
                newAppUI.Add(appUI)

                If appLocation.X + appSize.Width * 2 > AppAreaLocation.X + AppArea.Width Then
                    appLocation = New Point(AppAreaLocation.X, appLocation.Y + appSize.Height)
                Else
                    appLocation.Offset(appSize.Width, 0)
                End If
            Next
            day.ChildElements.AddRange(newAppUI)

            Dim sel As AppointmentSelectedUIElement = TryCast(parent, AppointmentSelectedUIElement)
            If sel IsNot Nothing Then
                Return True
            End If
            Return False
        ElseIf TypeOf parent Is EditorWithTextDisplayTextUIElement Then
            Dim editor As EditorWithTextDisplayTextUIElement = DirectCast(parent, EditorWithTextDisplayTextUIElement)
            editor.WrapText = True
            editor.MultiLine = True

            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(editor.Text, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 24, CInt(objSizeF.Height))

            If iHeight + 4 > editor.Parent.Parent.Parent.Rect.Height OrElse editor.Parent.Parent.Parent.Rect.Height < 24 Then
                iHeight = editor.Parent.Parent.Parent.Rect.Height - 4
            End If

            Dim appSize As Size
            If editor.Parent.Parent.Parent.Rect.Width < 95 Then
                appSize = New Size(editor.Parent.Parent.Parent.Rect.Width - (editor.Parent.Parent.Parent.Rect.Width * 0.2), iHeight)
            Else
                appSize = New Size(IIf(objSizeF.Width > 90, 90, objSizeF.Width), iHeight)
            End If

            Dim iX As Integer = editor.Parent.Parent.Parent.Rect.X
            editor.Rect = New Rectangle(editor.Parent.Parent.Parent.Rect.X + (editor.Parent.Parent.Parent.Rect.Width / 2) - (editor.Rect.Width / 2) + 4, editor.Rect.Y, appSize.Width, iHeight)
        End If
        Return False
    End Function

End Class

Class MonthViewSingleAppointmentDrawFilter
    Implements IUIElementDrawFilter
    Private DayApps As Dictionary(Of Appointment, SingleDayAppointmentUIElement)
    Public Sub New(ByRef DayApps As Dictionary(Of Appointment, SingleDayAppointmentUIElement))
        Me.DayApps = DayApps
    End Sub

    Public Function DrawElement(ByVal drawPhase As DrawPhase, ByRef drawParams As UIElementDrawParams) As Boolean Implements IUIElementDrawFilter.DrawElement
        Return True
    End Function

    Public Function GetPhasesToFilter(ByRef drawParams As UIElementDrawParams) As DrawPhase Implements IUIElementDrawFilter.GetPhasesToFilter
        Dim phase As DrawPhase = DrawPhase.None
        If TypeOf drawParams.Element Is SingleDayAppointmentUIElement Then
            Dim app As SingleDayAppointmentUIElement = TryCast(drawParams.Element, SingleDayAppointmentUIElement)

            If app.Appointment IsNot Nothing AndAlso DayApps.Count > 0 AndAlso Not DayApps(app.Appointment).Equals(app) Then
                phase = DrawPhase.BeforeDrawElement
            End If
        End If
        Return phase
    End Function
End Class

Public Class DayViewAppointmentCreationFilter
    Implements IUIElementCreationFilter

    Public Sub AfterCreateChildElements(ByVal parent As UIElement) Implements IUIElementCreationFilter.AfterCreateChildElements
        If TypeOf parent Is Infragistics.Win.UltraWinSchedule.DayView.AppointmentUIElement Then
            Dim appointment As Infragistics.Win.UltraWinSchedule.Appointment = TryCast(parent.GetContext(GetType(Infragistics.Win.UltraWinSchedule.Appointment)), Infragistics.Win.UltraWinSchedule.Appointment)
            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(appointment.Subject, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 35, CInt(objSizeF.Height))
            parent.Rect = New Rectangle(parent.Rect.X, parent.Rect.Y, parent.Rect.Width, iHeight)
        ElseIf TypeOf parent Is EditorWithTextDisplayTextUIElement Then
            Dim editor As EditorWithTextDisplayTextUIElement = DirectCast(parent, EditorWithTextDisplayTextUIElement)
            editor.WrapText = True
            editor.MultiLine = True

            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(editor.Text, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 35, CInt(objSizeF.Height))
            editor.Rect = New Rectangle(editor.Rect.X, editor.Rect.Y, editor.Rect.Width, iHeight)
        End If
    End Sub

    Public Function BeforeCreateChildElements(ByVal parent As UIElement) As Boolean Implements IUIElementCreationFilter.BeforeCreateChildElements
        Return False
    End Function

End Class

#End Region